import sys
sys.path.append("../ei2c")
from ei2c import *
import ei2cdefine
import math

if __name__ == "__main__":

    # Get the IP, user and password for the ELMA crate from ei2cdefine.py
    ip   = ei2cdefine.HNAME
    user = ei2cdefine.USER
    pwd  = ei2cdefine.PWD
    testelma = EI2C(ip, user, pwd)
    testelma.open()

    # Ask for slot number
    while 1:
        try:
            slot = raw_input("Slot no.: ")
            slot = int(slot)
            break
        except TypeError as e:
            print("Please input a decimal slot number.")
        except SlotError as e:
            print(e.strerror)
        except KeyboardInterrupt:
            sys.exit();
        except:
            print("Unexpected error: ", sys.exc_info()[0])

    # Output to file?
    while 1:
        try:
            inp = raw_input("Output samples to file? (y/n) ").lower()
            if (inp  == 'y'):
                fname = raw_input("File name (with extension): ");
                f = open(fname, 'w')
                break
            elif (inp == 'n'):
                break
        except KeyboardInterrupt:
            print("Exiting on user request")
            sys.exit();
        except:
            print("Error: " + sys.exc_info()[0]);

    # Read while FIFO is not empty
    csr   = testelma.read(slot, 0x3c)
    empty = csr & (1 << 17)
    used  = csr & 0x7f

    chcnt = [0, 0, 0, 0, 0, 0]

    while not empty:
        # Get channel mask from meta register and convert it to channel number
        ch = testelma.read(slot, 0x2c) & 0x3f

        # Increment channel counters if channel bit is set in FIFO
        i = 0
        if (ch & (1 << i)):
            chcnt[i] += 1
        i += 1
        if (ch & (1 << i)):
            chcnt[i] += 1
        i += 1
        if (ch & (1 << i)):
            chcnt[i] += 1
        i += 1
        if (ch & (1 << i)):
            chcnt[i] += 1
        i += 1
        if (ch & (1 << i)):
            chcnt[i] += 1
        i += 1
        if (ch & (1 << i)):
            chcnt[i] += 1

        #ch = 1 + int(math.log(ch)/math.log(2))

        # Read cycles and seconds counters
        cyc = testelma.read(slot, 0x30)
        sl  = testelma.read(slot, 0x34)
        sh  = testelma.read(slot, 0x38)

        # The seconds counter indicates actual seconds; the cycles counter
        # indicates 8-ns cycles, so convert to ns
        s   = (sh << 32) | sl
        ns  = cyc*8

        # And print
        csr   = testelma.read(slot, 0x3c)
        empty = csr & (1 << 17)
        used  = csr & 0x7f
        sample = "(%3d) %s : %d.%09d sec" % (used, "{0:#08b}".format(ch)[2:], s, ns)
        if 'f' in locals():
            f.write(sample + '\n');
        else:
            print(sample)

    # Number of pulses are always printed to console
    print("Number of pulses:")
    i = 0
    print("CH%d: %d" % (i+1, chcnt[i]))
    i += 1
    print("CH%d: %d" % (i+1, chcnt[i]))
    i += 1
    print("CH%d: %d" % (i+1, chcnt[i]))
    i += 1
    print("CH%d: %d" % (i+1, chcnt[i]))
    i += 1
    print("CH%d: %d" % (i+1, chcnt[i]))
    i += 1
    print("CH%d: %d" % (i+1, chcnt[i]))
    i += 1

    if 'f' in locals():
        f.close()
