import sys
sys.path.append("../ei2c")
from ei2c import *
import ei2cdefine

MB_BASE = 0x40
MB_CR_OFS = 0x00
MB_SR_OFS = 0x04
MB_GBBAR_OFS = 0x08
MB_MBBAR_OFS = 0x0C
MB_FAR_OFS   = 0x10

def spi_transfer(nbytes, cs, dat):
  retval = 0
  wval = []

  # control byte, to be shifted left by 24
  ctrl = ((cs << 3) | 0x4) | (nbytes-1)

  # Use appropriate command by type
  #
  # write - we send up to three data bytes in one FAR register write
  # writemregs - we send up to 24 data bytes in eight FAR register writes
  if isinstance(dat,int):
    testelma.write(slot, MB_BASE+MB_FAR_OFS, (ctrl << 24) | dat)
  else:
    for i in xrange(len(dat)):
      wval.append((ctrl << 24) | dat[i])
    testelma.writemregs(slot, MB_BASE+MB_FAR_OFS, wval)

  # Read the data and prepare the return value
  while (retval & (1 << 28) == 0):
    retval = testelma.read(slot, MB_BASE+MB_FAR_OFS)

  return retval & 0xFFFFFF

def flash_read(addr, nrbytes):
  ret = []
  spi_transfer(1,1,0x0b)

  # send address in reverse order
  addr = ((addr & 0xff0000) >> 16) | (((addr & 0xff00) >> 8) << 8) | ((addr & 0xff) << 16)
  spi_transfer(3,1,addr)
  spi_transfer(1,1,0)

  # Read bytes in groups of three
  for i in range(nrbytes):
    ret.append(spi_transfer(1,1,0))
  spi_transfer(1,0,0)

  return ret

if __name__ == "__main__":

  # Get the IP, user and password for the ELMA crate from ei2cdefine.py
  ip   = ei2cdefine.HNAME
  user = ei2cdefine.USER
  pwd  = ei2cdefine.PWD
  testelma = EI2C(ip, user, pwd)
  testelma.open()

  # Ask for slot number
  while 1:
    try:
      slot = raw_input("Slot no.: ")
      slot = int(slot)
      break
    except TypeError as e:
      print("Please input a decimal slot number.")
    except SlotError as e:
      print(e.strerror)
    except KeyboardInterrupt:
      sys.exit();
    except:
      print("Unexpected error: ", sys.exc_info()[0])

  # Get start address
  while 1:
    try:
      sa = int(raw_input("Start address : 0x"), 16)
      break
    except ValueError as e:
      print("Please input a hexadecimal address!")
    except KeyboardInterrupt:
      print("KeyboardInterrupt")
      sys.exit()
    except:
      print("Unexpected error: ", sys.exc_info()[0])

  # Get end address
  while 1:
    try:
      ea = int(raw_input("End address   : 0x"), 16)
      break
    except ValueError as e:
      print("Please input a hexadecimal address!")
    except KeyboardInterrupt:
      print("KeyboardInterrupt")
      sys.exit()
    except:
      print("Unexpected error: ", sys.exc_info()[0])

  # Start reading
  dat = []
  for i in range(sa, ea, 256):
    dat += flash_read(i, 256)
  dat = ''.join(map(chr,dat))
  f = open("flash.bin", "wb")
  f.write(dat)
  f.close()
  testelma.close()

  print("DONE!")
  print("Output stored to flash.bin")
